<?php

// =============================================================================
// Inline Tweets
//
// Released under the GNU General Public Licence v2
// http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
//
// Please refer all questions/requests to: mdjekic@gmail.com
//
// This is an add-on for WordPress
// http://wordpress.org/
// =============================================================================

// =============================================================================
// This piece of software is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.
// =============================================================================

/*
  Plugin Name: Inline Tweets
  Plugin URI: http://projects.djekic.net/inline-tweets
  Description: Embed latest tweet(s) or a specific tweet in any page, post or text widget and apply your custom styles.
  Version: 1.0
  Author: Miloš Đekić
  Author URI: http://milos.djekic.net
 */

/**
 * Returns the error HTML entity
 *
 * @param string $error text to be displayed as error
 * @return string HTML
 */
function inline_tweets_inline_error($error = "Error") {
    return '<div class="inline_tweets_error">'.$error.'</div>';
}

/**
 * Embeds tweet into an HTML entity
 *
 * @param $tweet
 * @return string HTML
 */
function inline_tweets_embed_tweet($tweet) {
    // get tweet content
    $content = $tweet['text'];

    // append tweet author
    $content = '<a class="inline_tweets_author" href="http://twitter.com/'.$tweet['user']['screen_name'].'" target="_blank">'."@{$tweet['user']['screen_name']}".'</a> ' . $content;

    // render tags
    foreach($tweet["entities"]["hashtags"] as $tag) {
        $content = str_replace("#{$tag['text']}",'<span class="inline_tweets_tag">'."#{$tag['text']}".'</span>',$content);
    }

    // render URLs
    foreach($tweet["entities"]["urls"] as $url) {
        $content = str_replace($url["url"],'<a class="inline_tweets_url" href="'.$url["expanded_url"].'" target="_blank">'.$url["url"].'</a>',$content);
    }

    // render mentions
    foreach($tweet["entities"]["user_mentions"] as $mention) {
        $content = str_replace("@{$mention['screen_name']}",'<a class="inline_tweets_mention" href="http://twitter.com/'.$mention['screen_name'].'" target="_blank">'."@{$mention['screen_name']}".'</a>',$content);
    }

    // add timestamp
    $content .= ' <span class="inline_tweets_timestamp">' . date('M d',strtotime($tweet['created_at'])) . '</span>';

    // return rendered tweet content
    return $content;
}

/**
 * Performs a curl request to Twitter API to fetch tweet(s)
 *
 * @param $url
 * @return mixed
 */
function inline_tweets_curl($url) {
    // init curl
    $ch = curl_init();

    // parametrize curl calls
    curl_setopt($ch, CURLOPT_URL,$url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

    // get result
    $result = curl_exec($ch);

    // close curl stream
    curl_close($ch);

    // return result
    return $result;
}

/**
 * Request handler intercepts requests to admin.php and checks if any actions
 * should be performed ('inline_tweets_action' parameter is in the request array)
 */
function inline_tweets_request_handler() {
    // check if inline_tweets action is to be performed
    if(empty($_REQUEST['inline_tweets_action'])) return;

    // create HTML content
    $html = inline_tweets_handle($_REQUEST['inline_tweets_action']);

    // send response
    header('Content-type: application/json');
    echo json_encode(array('html' => $html));
    die;
}

/**
 * Handles an inline tweet
 *
 * @param $inline_tweet [account:action:param]
 * @return String handled inline tweet
 */
function inline_tweets_handle($inline_tweet) {
    // clear tags
    $inline_tweet = str_replace('[tweet]','',$inline_tweet);
    $inline_tweet = str_replace('[/tweet]','',$inline_tweet);

    // get embed data
    $embed_data = explode(':',$inline_tweet);

    // check data structure
    if(count($embed_data) < 2) return inline_tweets_inline_error("Invalid structure.");

    // check action
    switch($embed_data[0]) {
        case 'latest':
            // check structure
            if(count($embed_data) < 3) return inline_tweets_inline_error("Invalid structure.");

            // determine screen name
            $screen_name = $embed_data[1];

            // determine count
            $count = intval($embed_data[2]);

            // check count
            if($count == 0) return inline_tweets_inline_error("Invalid tweets count");

            // get response from Twitter API
            $response = inline_tweets_curl("https://api.twitter.com/1/statuses/user_timeline.json"
                ."?include_entities=true&include_rts=true&screen_name={$screen_name}&count={$count}");

            // decode from JSON to array
            $data = @json_decode($response,true);

            // check data and return inline error if data is false
            if($data == false) return inline_tweets_inline_error("Unable to fetch tweet(s)");

            // start creating content
            $content = ($count == 1) ? '' : '<ul class="inline_tweets_ul">';

            // go trough all fetched tweets
            foreach($data as $tweet) {
                // input embedded tweet
                $content .= ($count == 1) ? '<div class="inline_tweet">' : '<li class="inline_tweet inline_tweets_li">';
                $content .= inline_tweets_embed_tweet($tweet);
                $content .= ($count == 1) ? '</div>' : '</li>';
            }

            // finish content
            $content .= ($count == 1) ? '' : '</ul>';

            // return created content
            return $content;
        case 'status':
            // determine status id
            $id = ($embed_data[1]);

            // get response from Twitter API
            $response = inline_tweets_curl("https://api.twitter.com/1/statuses/show.json?id={$id}&include_entities=true");

            // decode from JSON to array
            $data = @json_decode($response,true);

            // check data and return inline error if data is false
            if($data == false) return inline_tweets_inline_error("Unable to fetch tweet.");

            // return tweet
            return '<div class="inline_tweet">' . inline_tweets_embed_tweet($data) . '</div>';
    }

    // return inline error
    return inline_tweets_inline_error();
}

/**
 * Renders posts/pages content by changing placeholders with tweets
 *
 * @param $content content to be rendered
 * @return String rendered content
 */
function inline_tweets_render($content) {
    // define regex for searching for inline tweets
    $regex = '/\[tweet\](.*?)\[\/tweet\]/ism';

    // search for inline tweets
    $count = preg_match_all($regex,$content,$inline_tweets);

    // do nothing if there were no matches
    if($count == 0) return $content;

    // handle all matches
    foreach($inline_tweets as $inline_tweets_data) {
        // wrap inline tweet in placeholder div
        $content = str_replace($inline_tweets_data[0],'<div class="inline_tweets_placeholder">'.$inline_tweets_data[0].'</div>',$content);
    }

    // return content
    return $content;
}

/**
 * Loads Inline Tweets script in header
 */
function inline_tweets_load_js_in_header() {
    // load basic css styles
    echo '<style type="text/css">div.inline_tweets_placeholder{text-align: center;}</style>\n';

    // set url param for the loader image
    echo '<script type="text/javascript">var INLINE_TWEETS_LOADER_URL = "' . plugins_url('/loader.gif',__FILE__) . '";</script>\n';

    // load handling script
    wp_enqueue_script('inline_tweets', plugins_url('/inline_tweets.js',__FILE__));
}

// register request handler
add_action('init', 'inline_tweets_request_handler', 9999);

// add rendering filter for posts/pages
add_filter('the_content','inline_tweets_render');

// add rendering filter for text widgets content
add_filter('widget_text','inline_tweets_render');

// add head action to include inline-tweets script
add_action('wp_head', 'inline_tweets_load_js_in_header');